# -*- coding: utf-8 -*-
from django import forms
from django.forms.widgets import Input, TextInput
from mptt.forms import TreeNodeChoiceField
from geo.models import AdministrativeAreaType, AdministrativeArea, Country, Location

class CountryForm(forms.ModelForm):
    class Meta:
        model = Country
        widgets = {
            'fullname': TextInput(attrs={'size': 100}),
            'iso_code': TextInput(attrs={'size': 2}),
            'iso3_code': TextInput(attrs={'size': 3}),
            'num_code': TextInput(attrs={'size': 5}),
#            'tld': TextInput(attrs={'size': 5}),
            }

class AreaForm(forms.ModelForm):
    class Meta:
        model = AdministrativeArea
        widgets = {
            'name': TextInput(attrs={'size': 100}),
            'code': TextInput(attrs={'size': 3}),
            }

class AdministrativeAreaTypeForm(forms.ModelForm):

    for_country = None
    def __init__(self, *args, **kwargs):
        super(AdministrativeAreaTypeForm, self).__init__(*args, **kwargs)
        for_country = kwargs.pop('for_country', self.for_country)
        if 'initial' in kwargs:
            self.fields['type'].queryset = AdministrativeAreaType.objects.filter(country=initial.country)
        elif for_country:
            self.fields['type'].queryset = AdministrativeAreaType.objects.filter(country=for_country)

    class Meta:
        model = AdministrativeArea
        widgets = {
            'code': TextInput(attrs={'size': 10}),
            }


class LocationForm(forms.ModelForm):
    area = TreeNodeChoiceField(AdministrativeArea.objects.all(), required=False)

    class Meta:
        model = Location
        widgets = {
            'code': TextInput(attrs={'size': 10}),

            }

def administrativeareatypeform_factory_for_country(country):
    """ returns a AdministrativeAreaForm type for a specific Country
        ie. only AdministrativeAreaType related to the Counry are allowed.
        used by the admin's inlines

    """
    if country is None:
        return AdministrativeAreaTypeForm
    name = str('%sAdministrativeAreaForm' % country.iso_code)
    args = {'for_country':country}
    return type(name, (AdministrativeAreaTypeForm,), args)



"""
GeoNames city data import script.
Requires the following files:

http://download.geonames.org/export/dump/
- Countries:            countryInfo.txt
- Regions:              admin1CodesASCII.txt
- Subregions:           admin2Codes.txt
- Cities:               cities5000.zip
- Districts:            hierarchy.zip
- Localization:         alternateNames.zip

http://download.geonames.org/export/zip/
- Postal Codes:         allCountries.zip
"""

import os
import urllib
import logging
import zipfile
import time
from collections import defaultdict
from itertools import chain
from optparse import make_option
from django.core.management.base import BaseCommand
from django.template.defaultfilters import slugify
from django.db import connection
from django.contrib.gis.gdal.envelope import Envelope
from ...conf import *
from ...models import *
from ...util import geo_distance

class Command(BaseCommand):
    app_dir = os.path.normpath(os.path.dirname(os.path.realpath(__file__)) + '/../..')
    data_dir = os.path.join(app_dir, 'data')
    logger = logging.getLogger("cities")

    option_list = BaseCommand.option_list + (
        make_option('--force', action='store_true', default=False,
            help='Import even if files are up-to-date.'
        ),
        make_option('--import', metavar="DATA_TYPES", default='all',
            help =  'Selectively import data. Comma separated list of data types: '
            + str(import_opts).replace("'",'')
        ),
        make_option('--flush', metavar="DATA_TYPES", default='',
            help =  "Selectively flush data. Comma separated list of data types."
        ),
        )

    def handle(self, *args, **options):
        self.download_cache = {}
        self.options = options

        self.force = self.options['force']

        self.flushes = [e for e in self.options['flush'].split(',') if e]
        if 'all' in self.flushes: self.flushes = import_opts_all
        for flush in self.flushes:
            func = getattr(self, "flush_" + flush)
            func()

        self.imports = [e for e in self.options['import'].split(',') if e]
        if 'all' in self.imports: self.imports = import_opts_all
        if self.flushes: self.imports = []
        for import_ in self.imports:
            func = getattr(self, "import_" + import_)
            func()

    def call_hook(self, hook, *args, **kwargs):
        if hasattr(settings, 'plugins'):
            for plugin in settings.plugins[hook]:
                try:
                    func = getattr(plugin,hook)
                    func(self, *args, **kwargs)
                except HookException as e:
                    error = str(e)
                    if error: self.logger.error(error)
                    return False
        return True

    def download(self, filekey):
        filename = settings.files[filekey]['filename']
        web_file = None
        urls = [e.format(filename=filename) for e in settings.files[filekey]['urls']]
        for url in urls:
            try:
                web_file = urllib.urlopen(url)
                if 'html' in web_file.headers['content-type']: raise Exception()
                break
            except:
                web_file = None
                continue
        else:
            self.logger.error("Web file not found: {}. Tried URLs:\n{}".format(filename, '\n'.join(urls)))

        uptodate = False
        filepath = os.path.join(self.data_dir, filename)
        if web_file is not None:
            web_file_time = time.strptime(web_file.headers['last-modified'], '%a, %d %b %Y %H:%M:%S %Z')
            web_file_size = int(web_file.headers['content-length'])
            if os.path.exists(filepath):
                file_time = time.gmtime(os.path.getmtime(filepath))
                file_size = os.path.getsize(filepath)
                if file_time >= web_file_time and file_size == web_file_size:
                    self.logger.info("File up-to-date: " + filename)
                    uptodate = True
        else:
            self.logger.warning("Assuming file is up-to-date")
            uptodate = True

        if not uptodate and web_file is not None:
            self.logger.info("Downloading: " + filename)
            if not os.path.exists(self.data_dir):
                os.makedirs(self.data_dir)
            file = open(os.path.join(self.data_dir, filename), 'wb')
            file.write(web_file.read())
            file.close()
        elif not os.path.exists(filepath):
            raise Exception("File not found and download failed: " + filename)

        return uptodate

    def download_once(self, filekey):
        if filekey in self.download_cache: return self.download_cache[filekey]
        uptodate = self.download_cache[filekey] = self.download(filekey)
        return uptodate

    def get_data(self, filekey):
        filename = settings.files[filekey]['filename']
        file = open(os.path.join(self.data_dir, filename), 'rb')
        name, ext = filename.rsplit('.',1)
        if (ext == 'zip'):
            zip = zipfile.ZipFile(file)
            data = zip.read(name + '.txt').split('\n')
            zip.close()
        else:
            data = file.read().split('\n')
        file.close()
        return data

    def parse(self, data):
        for line in data:
            if len(line) < 1 or line[0] == '#': continue
            items = [e.strip() for e in line.split('\t')]
            yield items

    def import_country(self):
        uptodate = self.download('country')
        if uptodate and not self.force: return
        data = self.get_data('country')

        self.logger.info("Importing country data")
        for items in self.parse(data):
            if not self.call_hook('country_pre', items): continue

            country = Country()
            try: country.id = int(items[16])
            except: continue
            country.name = items[4]
            country.slug = slugify(country.name)
            country.code = items[0]
            country.population = items[7]
            country.continent = items[8]
            country.tld = items[9][1:] # strip the leading .

            if not self.call_hook('country_post', country, items): continue
            country.save()
            self.logger.debug("Added country: {}, {}".format(country.code, country))

    def import_region_common(self, region, items):
        class_ = region.__class__
        region.id = int(items[3])
        region.name = items[2]
        region.name_std = items[1]
        region.slug = slugify(region.name)
        region.code = items[0]

        # Find country
        country_code = region.code.split('.')[0]
        try: region.country = self.country_index[country_code]
        except:
            self.logger.warning("{}: {}: Cannot find country: {} -- skipping".format(class_.__name__, region.name, country_code))
            return None

        return region

    def build_country_index(self):
        if hasattr(self, 'country_index'): return

        self.logger.info("Building country index")
        self.country_index = {}
        for obj in Country.objects.all():
            self.country_index[obj.code] = obj

    def import_region(self):
        uptodate = self.download('region')
        if uptodate and not self.force: return
        data = self.get_data('region')

        self.build_country_index()

        self.logger.info("Importing region data")
        for items in self.parse(data):
            if not self.call_hook('region_pre', items): continue

            region = self.import_region_common(Region(), items)
            if not region: continue

            if not self.call_hook('region_post', region, items): continue
            region.save()
            self.logger.debug("Added region: {}, {}".format(region.code, region))

    def build_region_index(self):
        if hasattr(self, 'region_index'): return

        self.logger.info("Building region index")
        self.region_index = {}
        for obj in chain(Region.objects.all(), Subregion.objects.all()):
            self.region_index[obj.code] = obj

    def import_subregion(self):
        uptodate = self.download('subregion')
        if uptodate and not self.force: return
        data = self.get_data('subregion')

        self.build_country_index()
        self.build_region_index()

        self.logger.info("Importing subregion data")
        for items in self.parse(data):
            if not self.call_hook('subregion_pre', items): continue

            subregion = self.import_region_common(Subregion(), items)
            if not subregion: continue

            # Find region
            level = Region.levels.index("subregion") - 1
            region_code = '.'.join(subregion.code.split('.')[:level+2])
            try: subregion.region = self.region_index[region_code]
            except:
                self.logger.warning("Subregion: {}: Cannot find region: {}".format(subregion.name, region_code))
                continue

            if not self.call_hook('subregion_post', subregion, items): continue
            subregion.save()
            self.logger.debug("Added subregion: {}, {}".format(subregion.code, subregion))

        del self.region_index

    def import_city_common(self, city, items):
        class_ = city.__class__
        city.id = int(items[0])
        city.name = items[2]
        city.name_std = items[1]
        city.slug = slugify(city.name)
        city.location = Point(float(items[5]), float(items[4]))
        city.population = items[14]

        # Find country
        country = None
        country_code = items[8]
        try: country = self.country_index[country_code]
        except:
            self.logger.warning("{}: {}: Cannot find country: {} -- skipping".format(class_.__name__, city.name, country_code))
            return None
        if class_ is City: city.country = country

        # Find region, search highest level first
        item_offset = 10
        for level, level_name in reversed(list(enumerate(Region.levels))):
            if not items[item_offset+level]: continue
            try:
                code = '.'.join([country_code] + [items[item_offset+i] for i in range(level+1)])
                region = self.region_index[code]
                if class_ is City:
                    setattr(city, level_name, region)
            except:
                self.logger.log(logging.DEBUG if level else logging.WARNING, # Escalate if level 0 failed
                    "{}: {}: Cannot find {}: {}".format(class_.__name__, city.name, level_name, code))


        return city

    def import_city(self):
        uptodate = self.download_once('city')
        if uptodate and not self.force: return
        data = self.get_data('city')

        self.build_country_index()
        self.build_region_index()

        self.logger.info("Importing city data")
        for items in self.parse(data):
            if not self.call_hook('city_pre', items): continue

            type = items[7]
            if type not in city_types: continue

            city = self.import_city_common(City(), items)
            if not city: continue

            if not self.call_hook('city_post', city, items): continue
            city.save()
            self.logger.debug("Added city: {}".format(city))

    def build_hierarchy(self):
        if hasattr(self, 'hierarchy'): return

        self.download('hierarchy')
        data = self.get_data('hierarchy')

        self.logger.info("Building hierarchy index")
        self.hierarchy = {}
        for items in self.parse(data):
            parent_id = int(items[0])
            child_id = int(items[1])
            self.hierarchy[child_id] = parent_id

    def import_district(self):
        uptodate = self.download_once('city')
        if uptodate and not self.force: return
        data = self.get_data('city')

        self.build_country_index()
        self.build_region_index()
        self.build_hierarchy()

        self.logger.info("Building city index")
        city_index = {}
        for obj in City.objects.all():
            city_index[obj.id] = obj

        self.logger.info("Importing district data")
        for items in self.parse(data):
            if not self.call_hook('district_pre', items): continue

            type = items[7]
            if type not in district_types: continue

            district = self.import_city_common(District(), items)
            if not district: continue

            # Find city
            city = None
            try: city = city_index[self.hierarchy[district.id]]
            except:
                self.logger.warning("District: {}: Cannot find city in hierarchy, using nearest".format(district.name))
                city_pop_min = 100000
                if connection.ops.mysql:
                    # mysql doesn't have distance function, get nearest city within 2 degrees
                    search_deg = 2
                    min_dist = float('inf')
                    bounds = Envelope(  district.location.x-search_deg, district.location.y-search_deg,
                        district.location.x+search_deg, district.location.y+search_deg)
                    for e in City.objects.filter(population__gt=city_pop_min).filter(location__intersects=bounds.wkt):
                        dist = geo_distance(district.location, e.location)
                        if dist < min_dist:
                            min_dist = dist
                            city = e
                else:
                    city = City.objects.filter(population__gt=city_pop_min).distance(district.location).order_by('distance')[0]
            if not city:
                self.logger.warning("District: {}: Cannot find city -- skipping".format(district.name))
                continue
            district.city = city

            if not self.call_hook('district_post', district, items): continue
            district.save()
            self.logger.debug("Added district: {}".format(district))

    def import_alt_name(self):
        uptodate = self.download('alt_name')
        if uptodate and not self.force: return
        data = self.get_data('alt_name')

        self.logger.info("Building geo index")
        geo_index = {}
        for type_ in geo_alt_names:
            for obj in type_.objects.all():
                geo_index[obj.id] = {
                    'type': type_,
                    'object': obj,
                    }

        self.logger.info("Importing alternate name data")
        for items in self.parse(data):
            if not self.call_hook('alt_name_pre', items): continue

            # Only get names for languages in use
            locale = items[2]
            if not locale: locale = 'und'
            if not locale in settings.locales: continue

            # Check if known geo id
            geo_id = int(items[1])
            try: geo_info = geo_index[geo_id]
            except: continue

            alt_type = geo_alt_names[geo_info['type']][locale]
            alt = alt_type()
            alt.id = int(items[0])
            alt.geo = geo_info['object']
            alt.name = items[3]
            alt.is_preferred = items[4]
            alt.is_short = items[5]

            if not self.call_hook('alt_name_post', alt, items): continue
            alt.save()
            self.logger.debug("Added alt name: {}, {} ({})".format(locale, alt, alt.geo))

    def import_postal_code(self):
        uptodate = self.download('postal_code')
        if uptodate and not self.force: return
        data = self.get_data('postal_code')

        self.build_country_index()
        self.build_region_index()

        self.logger.info("Importing postal codes")
        for items in self.parse(data):
            if not self.call_hook('postal_code_pre', items): continue

            country_code = items[0]
            if country_code not in settings.postal_codes: continue

            # Find country
            code = items[1]
            country = None
            try:
                country = self.country_index[country_code]
            except:
                self.logger.warning("Postal code: {}: Cannot find country: {} -- skipping".format(code, country_code))
                continue

            pc = PostalCode()
            pc.country = country
            pc.code = code
            pc.name = items[2]
            try:
                pc.region = country.region_set.get(name=items[3])
                pc.subregion = pc.region.subregion_set.get(name=items[5])
            except:
                pass

            try:
                pc.location = Point(float(items[10]), float(items[9]))
            except:
                self.logger.warning("Postal code: {}, {}: Invalid location ({}, {})".format(pc.country, pc.code, items[10], items[9]))
                pc.location = Point(0,0)

            if not self.call_hook('postal_code_post', pc, items): continue
            pc.save()
            self.logger.debug("Added postal code: {}, {}".format(pc.country, pc))

    def flush_country(self):
        self.logger.info("Flushing country data")
        Country.objects.all().delete()

    def flush_region(self):
        self.logger.info("Flushing region data")
        Region.objects.all().delete()

    def flush_subregion(self):
        self.logger.info("Flushing subregion data")
        Subregion.objects.all().delete()

    def flush_city(self):
        self.logger.info("Flushing city data")
        City.objects.all().delete()

    def flush_district(self):
        self.logger.info("Flushing district data")
        District.objects.all().delete()

    def flush_alt_name(self):
        self.logger.info("Flushing alternate name data")
        [geo_alt_name.objects.all().delete() for locales in geo_alt_names.values() for geo_alt_name in locales.values()]

    def flush_postal_code(self):
        self.logger.info("Flushing postal code data")
        [postal_code.objects.all().delete() for postal_code in postal_codes.values()]



# -*- coding: utf-8 -*-
'''
Created on May 7, 2010

@author: sax
'''
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.validators import MinLengthValidator
from django.db import models
from django.db.models.manager import Manager
from django.utils.translation import ugettext_lazy as _
import logging
from mptt.exceptions import InvalidMove
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey

logger = logging.getLogger("geo")


class Currency(models.Model):
    code = models.CharField(max_length=5, unique=True, help_text="ISO 4217 code")
    name = models.CharField(max_length=30)
    symbol = models.CharField(max_length=5, blank=True, null=True)

    class Meta:
        app_label = 'geo'
        ordering = ['code', ]

    def __unicode__(self):
        return unicode("%s (%s)" % (self.code, self.name))


CONTINENTS = (
    ('AF', _('Africa')),
    ('AS', _('Asia')),
    ('EU', _('Europe')),
    ('NA', _('North America')),
    ('SA', _('South America')),
    ('OC', _('Oceania')),
    ('AN', _('Antartica'))
    )

Regions = zip(range(1, 5), ('Africa', 'Americas', 'Asia', 'Middle East'))

class CountryManager(Manager):
    use_for_related_fields = True

    def _by_continent(self, c):
        return self.get_query_set().filter(continent=c)

    def asia(self):
        return self._by_continent('AS')

    def europa(self):
        return self._by_continent('EU')

    def north_america(self):
        return self._by_continent('NA')

    def south_america(self):
        return self._by_continent('SA')

    def oceania(self):
        return self._by_continent('OC')

    def antartica(self):
        return self._by_continent('AN')

    def get_by_natural_key(self, iso_code):
        return self.get(iso_code=iso_code)


class Country(models.Model):
    """ Model for the country of origin """
    iso_code = models.CharField(max_length=2, unique=True, blank=False, null=False, db_index=True,
        help_text='ISO 3166-1 alpha 2', validators=[MinLengthValidator(2)])
    iso3_code = models.CharField(max_length=3, blank=False, null=False, db_index=True,
        help_text='ISO 3166-1 alpha 3', validators=[MinLengthValidator(3)])
    num_code = models.CharField(max_length=3, blank=True, null=True, help_text='ISO 3166-1 numeric')
    name = models.CharField(max_length=100, db_index=True)
    fullname = models.CharField(max_length=100, db_index=True)

    region = models.IntegerField(choices=Regions, blank=True, null=True)
    continent = models.CharField(choices=CONTINENTS, max_length=2)
    currency = models.ForeignKey(Currency, blank=True, null=True)

    #    tld = models.CharField(max_length=6, blank=True, null=True, help_text='internet tld')
    #    tz = models.IntegerField(blank=True, null=True, help_text='time zone')

    fullname.alphabetic_filter = True
    objects = CountryManager()

    class Meta:
        app_label = 'geo'
        verbose_name_plural = _('Countries')
        ordering = ['name']

    def __unicode__(self):
        return unicode(self.fullname)

    def full_clean(self, exclude=None):
        super(Country, self).full_clean(exclude)
        self.iso_code = self.iso_code.upper()
        self.iso3_code = self.iso3_code.upper()

    def natural_key(self):
        return (self.iso_code,)

    def __contains__(self, item):
        if hasattr(item, 'country'):
            return item.country.iso_code == self.iso_code

    def cities(self):
        return self.location_set.cities()


class AdministrativeAreaTypeManager(TreeManager):
    use_for_related_fields = True

    def get_by_natural_key(self, iso_code, name):
        return self.get(country__iso_code=iso_code, name=name)


class AdministrativeAreaType(MPTTModel):
    name = models.CharField(_('Name'), max_length=100, db_index=True)
    country = models.ForeignKey(Country)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
    objects = AdministrativeAreaTypeManager()

    class Meta:
        verbose_name = _("Administrative Area Type")
        verbose_name_plural = _("Administrative Area Types")
        app_label = 'geo'
        ordering = ['name']
        order_with_respect_to = 'country'

    def __unicode__(self):
        return unicode(self.name)

    def natural_key(self):
        return self.country.natural_key() + (self.name, )

    natural_key.dependencies = ['geo.country']

    def clean(self):
        if self.parent == self:
            raise ValidationError(_('`%s` cannot contains same type') % self.parent)
        super(AdministrativeAreaType, self).clean()

    def get_or_create(self, **kwargs):
        return AdministrativeArea.objects.get_or_create(country=self.country, type=self, **kwargs)


class AdministrativeAreaManager(TreeManager):
    use_for_related_fields = True

    def get_by_natural_key(self, iso_code, name):
        return self.get(country__iso_code=iso_code, name=name)

    def get_or_create(self, **kwargs):
        return super(AdministrativeAreaManager, self).get_or_create(**kwargs)


class AdministrativeArea(MPTTModel):
    """ Administrative areas that can contains other AdministrativeArea and/or Location

    """

    name = models.CharField(_('Name'), max_length=255, db_index=True)
    code = models.CharField(_('Code'), max_length=10, blank=True, null=True, db_index=True, help_text='ISO 3166-2 code')
    parent = TreeForeignKey('self', null=True, blank=True, related_name='areas')
    country = models.ForeignKey(Country, related_name='areas')
    type = models.ForeignKey(AdministrativeAreaType)
    objects = AdministrativeAreaManager()

    class Meta:
        verbose_name = _("Administrative Area")
        verbose_name_plural = _("Administrative Areas")
        unique_together = (('name', 'country', 'type'),)
        app_label = 'geo'
        ordering = ['name']
        order_with_respect_to = 'country'

    def __unicode__(self):
        return unicode(self.name)

    def natural_key(self):
        return (self.country.iso_code, self.name )

    natural_key.dependencies = ['geo.country']

    def clean(self):
        if self.parent == self:
            raise ValidationError(_('`%s` cannot contains self') % self)
        if self.parent and self.parent.type == self.type:
            raise ValidationError(_('`%s` cannot contains same type') % self.parent.type)
        if (self.pk and self.parent) and self.parent in self:
            raise ValidationError(_('`%s` cannot contains `%s`') % (self, self.parent))

        super(AdministrativeArea, self).clean()

    def save(self, *args, **kwargs):
        if not self.country_id:
            self.country = self.parent.country
        self.clean()
        super(AdministrativeArea, self).save(*args, **kwargs)

    def __contains__(self, item):
        if isinstance(item, AdministrativeArea) and item.is_child_node():
            return item.is_descendant_of(self)
        elif isinstance(item, Location) and item.area:
            return item.area.is_descendant_of(self)


class LocationManager(models.Manager):
    use_for_related_fields = True

    def get_or_none(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except ObjectDoesNotExist:
            return None

    def cities(self):
        return self.get_query_set().filter(type=Location.CITY)

    def get_by_natural_key(self, country_iso_code, name, lat, lng):
        if lat == 'None':
            lat = None
        if lng == 'None':
            lng = None

        return self.get(country__iso_code=country_iso_code, name=name, lat=lat, lng=lng)


class Location(models.Model):
    """ Administrative location ( city, place everything with a name and Lat/Lng that
    is not intetend to contains anything ( use Areas for that
    """
    NONE = 0
    COUNTRY = 10
    EXACT = 20

    ACCURACY = (
        (NONE, _('None')),
        (COUNTRY, _('Country')),
        (EXACT, _('Exact')),
        )

    CITY = 0
    OTHER = 1
    TYPE = ((CITY, _("City")),
            (OTHER, _("Other")),
        )

    country = models.ForeignKey(Country, db_index=True)
    area = models.ForeignKey(AdministrativeArea, db_index=True, blank=True, null=True)
    type = models.IntegerField(choices=TYPE, default=CITY)
    is_capital = models.BooleanField(default=False,
        help_text="True if is the capital of `country`")

    is_administrative = models.BooleanField(default=False,
        help_text="True if is administrative for `area`")

    name = models.CharField(_('Name'), max_length=255, db_index=True)
    description = models.CharField(max_length=100, blank=True, null=True)
    lat = models.DecimalField(max_digits=18, decimal_places=12, blank=True, null=True)
    lng = models.DecimalField(max_digits=18, decimal_places=12, blank=True, null=True)
    acc = models.IntegerField(choices=ACCURACY, default=NONE, blank=True, null=True,
        help_text="Define the level of accuracy of lat/lng infos")

    objects = LocationManager()

    class Meta:
        verbose_name_plural = _('Locations')
        verbose_name = _('Location')
        app_label = 'geo'
        ordering = ('name', 'country')
        order_with_respect_to = 'country'

    def __unicode__(self):
        return unicode(self.name)

    def natural_key(self):
        return self.country.natural_key() + (self.name, str(self.lat), str(self.lng))

    natural_key.dependencies = ['geo.country']

    def clean(self):
        if self.area and self.area.country != self.country:
            raise ValidationError('Selected area not in selected country')
        super(Location, self).clean()




# -*- coding: utf-8 -*-
from django.contrib.admin.options import TabularInline
from django.contrib.admin.util import unquote
from django.core.urlresolvers import reverse
from geo.forms import CountryForm, AreaForm, administrativeareatypeform_factory_for_country, LocationForm
from geo.models import Country, Location, AdministrativeArea, AdministrativeAreaType, Currency
from geo.templatetags.geo import flag
from django.contrib.admin import ModelAdmin, site, TabularInline


def tabular_factory(model, fields=None, inline=None, form=None, **kwargs):
    """ factory for TabularInline

    >>> class MD(IModelAdmin):
    ...     inlines = [tabular_factory(Permission)]
    """
    attrs = {'model': model, 'fields': fields}
    read_only = kwargs.pop('read_only', False)
    if read_only:
        attrs['readonly_fields'] = fields
        attrs['can_delete'] = False

    Inline = inline or TabularInline
    name = "%sInLine" % model.__class__.__name__
    if form:
        attrs['form'] = form
    attrs.update(kwargs)
    Tab = type(name, (Inline,), attrs)
    return Tab


class ICurrency(ModelAdmin):
    search_fields = ('name', 'code')
    list_display = ('name', 'code', 'symbol', 'used_by')
    inlines = [tabular_factory(Country, fields=['name'], read_only=True)]

    def used_by(self, o):
        return ', '.join(['<a href="%s">%s</>' % (reverse('admin:geo_country_change', args=[c.pk]), c.name) for c in
                          Country.objects.filter(currency=o)])

    used_by.allow_tags = True


class AdministrativeAreaInline(TabularInline):
    model = AdministrativeArea

    def get_formset(self, request, obj=None, **kwargs):
        self.form = administrativeareatypeform_factory_for_country(obj)
        return super(AdministrativeAreaInline, self).get_formset(request, obj, **kwargs)


class ICountry(ModelAdmin):
    form = CountryForm
    search_fields = ('name', )
    list_display = ('name', 'continent', 'region', 'iso_code', 'iso3_code', 'currency', 'capital', 'flag')
    list_filter = ('continent', 'region', )
    cell_filter = ('continent', 'region', 'currency')
    fieldsets = [(None, {'fields': (('name', 'fullname'),
                                    ('iso_code', 'iso3_code', 'num_code'),
                                    ('region', 'continent', 'currency'),
                                    )})]
    inlines = (tabular_factory(Location, exclude=('description',)),
               AdministrativeAreaInline,
               tabular_factory(AdministrativeAreaType), )

    def flag(self, o):
        return flag(o)

    flag.allow_tags = True

    def capital(self, o):
        c = o.location_set.get_or_none(is_capital=True)
        if c:
            admin_url = reverse('admin:geo_location_change', args=[c.pk])
            return "<a href='%s'>%s</a>" % (admin_url, c.name)
        return c

    capital.allow_tags = True

    def change_view(self, request, object_id, form_url='', extra_context=None):
        obj = self.get_object(request, unquote(object_id))
        context = {'nodes': obj.areas.all()}
        context.update(extra_context or {})
        return super(ICountry, self).change_view(request, object_id, form_url, context)


class ILocation(ModelAdmin):
    search_fields = ('name', )
    list_display = ('name', 'country', 'area', 'is_administrative', 'is_capital')
    list_display_rel_links = cell_filter = ('country', 'area', 'is_administrative', 'is_capital')
    list_filter = ('is_administrative', 'is_capital')
    form = LocationForm

def rebuild_tree(modeladmin, request, queryset):
    modeladmin.model.objects.rebuild()
rebuild_tree.short_description = "Rebuild MPTT table structure"

class IArea(ModelAdmin):
    form = AreaForm
    search_fields = ('name', )
    list_display = ('name', 'parent', 'country', 'type', 'code')
    list_display_rel_links = cell_filter = ('country', 'type', 'code')
    list_filter = ('type', 'country')
    inlines = (tabular_factory(Location),)
    actions = [rebuild_tree,]

    def change_view(self, request, object_id, form_url='', extra_context=None):
        obj = self.get_object(request, unquote(object_id))
        context = {'nodes': obj.areas.all()}
        context.update(extra_context or {})
        return super(IArea, self).change_view(request, object_id, form_url, context)




class IAreaType(ModelAdmin):
    search_fields = ('name', )
    list_display = ('name', 'parent', 'country')
    list_display_rel_links = cell_filter = ('country', )
    list_filter = ('country', )
    inlines = (tabular_factory(AdministrativeArea),)
    actions = [rebuild_tree,]

    def change_view(self, request, object_id, form_url='', extra_context=None):
        obj = self.get_object(request, unquote(object_id))
        context = {'nodes': obj.children.all()}
        context.update(extra_context or {})
        return super(IAreaType, self).change_view(request, object_id, form_url, context)

    def rebuild_tree(self, request, queryset):
        self.model.objects.rebuild()
    rebuild_tree.short_description = "Rebuild MPTT table structure"


reg = ((Country, ICountry), (Location, ILocation), (AdministrativeArea, IArea),
       (AdministrativeAreaType, IAreaType), (Currency, ICurrency), )

for model, model_admin in reg:
    site.register(model, model_admin)
